@@ -10,6 +10,7 @@ from hexathon import (
 
 # local imports
 from chainlib.eth.tx import Tx
+from .src import Src
 
 
 def block_latest(id_generator=None):
@@ -76,7 +77,7 @@ def syncing(id_generator=None):
     return j.finalize(o)
 
 
-class Block(BaseBlock):
+class Block(BaseBlock, Src):
     """Encapsulates an Ethereum block
 
     :param src: Block representation data
@@ -87,7 +88,9 @@ class Block(BaseBlock):
     tx_generator = Tx
 
     def __init__(self, src):
-        self.hash = src['hash']
+        super(Block, self).__init__(src)
+        import sys
+        self.set_hash(src['hash'])
         try:
             self.number = int(strip_0x(src['number']), 16)
         except TypeError:
@@ -98,9 +101,10 @@ class Block(BaseBlock):
             self.timestamp = int(strip_0x(src['timestamp']), 16)
         except TypeError:
             self.timestamp = int(src['timestamp'])
+        self.author = src['author']
 
 
-    def get_tx(self, tx_hash):
+    def tx_index_by_hash(self, tx_hash):
         i = 0
         idx = -1
         tx_hash = add_0x(tx_hash)
@@ -117,4 +121,3 @@ class Block(BaseBlock):
         if idx == -1:
             raise AttributeError('tx {} not found in block {}'.format(tx_hash, self.hash))
         return idx
-
